:
# NAME:
#	apctl - APC tool
#
# SYNOPSIS:
#	apctl.sh -f "tty" [options] "cmd" ...
#
# DESCRIPTION:
#	This script talks to an APC Smart-UPS.  It started life as an
#	implementation of the 'ups_adjust' command for those of us who
#	run operating systems not supported by APC\'s power-chute
#	software.
#
#	Each "cmd" is sent to "tty" and responses read.
#
#	Options:
#
#	-f "tty"
#		Indicates the device that the UPS should be
#		monitored via. 
#
#	-v	Be verbose, output the variable name plus value.
#
#	-V "fs"
#		Separate variable name and value with "fs".
#
#	-p "prefix"
#		Prefix each variable name with "prefix".
#
#	-s "var"
#		Set "var" this requires a special syntax so is easier
#		to do this way.  If "var" starts with '+' or '-' we
#		step up or down as indicated.  Default is down.
#
#	Protocol:
#
#	Basically one sends a command character "c" to the UPS and it
#	responds accordingly.  For the most part a single character is
#	needed.   For those commands that represent eeprom variables,
#	the character "c" alone, retrieves the current value, while
#	the sequence 'Y' 'V' "c" '-' "c" is used to cycle the variable
#	to its next possible value.
#
#	Below are the command characters and/or sequences.  The
#	meaning of some are dubious.
#
#	Read-only:
#	
#	'Q'	Status.  A bitmask, bit 3 set if on-line, bit 4 if on
#		battery.
#
#	'f'	Battery capacity (%).
#
#	'F'	Frequency (Hz) input or outut?
#
#	'C'	Battery temperature (Celcius).
#
#	'B'	Battery voltage (Volts).
#
#	'L'	Utility voltage.
#
#	'M'	Utility max voltage.
#	
#	'N'	Utility min voltage.
#
#	'O'	Output voltage.
#	
#	'P'	UPS load. (%).
#
#	Read/write:
#	
#	'p'	UPS shutdown delay time (020,180,300,600) secs
#	
#	'r'	Wakeup delay time (0,60,180,300) secs.
#	
#	'e'	Wakeup capacity (0,10,25,90) %.
#	
#	's'	Line sensitivity (H,M,L).
#	
#	'q'	Low battery duration (2,5,7,10) min
#		when to signal low battery.
#		
#	'k'	Audible line failure alarm (0=imediate, T=30s after
#		fail, L=low battery, N=never).
#		
#	'o'	Output voltage on battery (225,230,240,220).
#	
#	'u'	High transfer pt. (253,264,271,280).
#	
#	'l'	Low transfer pt. (188,196,204,208).
#
#	'x'	Last battery replacement (mm/dd/yy).
#
#	Commands:
#
#	'A'	Test front panel LEDs.
#
#	'KKK'	Turn off UPS after delay controlled by 'p'.
#		Cancel with 0x7f.
#
#	'R'	Enter dumb mode (don't do this).
#
#	'S'	Soft shutdown.  Enter sleep mode after delay
#		controlled by 'p'.  When mains power restored, fire
#		everything up after the delays controlled by 'e' and 'r'.
#		This command appears only to be functional when
#		running on battery (or preceed with 'U').
#
#	'Y'	Enter smart mode.
#
#	'W'	Do a self test. UPS reports success failure?
#
#	'X'	Self test status?
#
#	'A'	Test LEDs.
#
#	'U'	Simulate a power failure.
#
#	The only special "cmd" is 'KKK' which is the sequence to tell
#	the UPS to turn itself off (after the delay controlled by
#	variable 'p').  The 'KKK' sequence must be sent with special
#	timming so is treated differently.  'S' is generally more
#	useful however.
#
#	When setting the read/write variables, each set command
#	toggles the value to the next position.  The resulting value
#	is the output.
#
#	There is lots more info available from
#	http://www.exploits.org/~rkroll/ups.html 
#
# BUGS:
#	The basic model of sending single character commands to the
#	UPS is described in the documentation for an old APC
#	Smart-UPS.  The command characters apart from those described
#	in that document were determined by frobbing the UPS via a
#	terminal so my interpretation may well be wrong in some cases.
#
#	The APC documentation suggests that a non-standard RS232 cable
#	is required to connect to the UPS.  In particular a High on
#	pin 1 for more than 4 seconds will shutdown the UPS.  My own
#	early efforts confirm this :-)  Play it safe and buy the
#	power-chute package \- just for the cable.  Be sure to use the
#	black cable for a Smart-UPS.
#	
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: apctl.sh,v 1.17 2021/10/04 07:06:58 sjg Exp $
#
#	@(#) Copyright (c) 1994-2021 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

# option list
opt_str=vf:s:V:p:b:

# option defaults if any
opt_f=/dev/tty02
opt_V==
# the apc ups only talks at 2400, but my 8 port serial card
# is stuck in overdrive (8 x normal speed)
opt_b=2400

# boilerplate
Myname=`basename $0 .sh`
Mydir=`dirname $0`

rc=.${Myname}rc
for d in $HOME $Mydir; do
  test -s $d/$rc && { . $d/$rc; break; }
done

PATH=$Mydir:$PATH

. setopts.sh

case `echo -n .` in -n*) N=; C="\c";; *) N=-n; C=;; esac

case "$opt_f" in
/dev/*) ;;
*)	opt_f=/dev/$opt_f;;
esac

TLK=/tmp/$Myname.$$
LK=/tmp/LCK.$Myname.`basename $opt_f`

trap "rm -f $TLK $LK" 0
trap "exit 0" 1 2 3 15

# make sure we don't trip over each other
echo $$ > $TLK
while :
do
  ln $TLK $LK 2>/dev/null && break
  lpid=`cat $LK`
  kill -0 $lpid 2>/dev/null || { rm -f $LK; continue; }
  sleep 5
done

test "x$opt_v" = x && s=1 || s=2
( stty $opt_b clocal cs8 -parenb -echo
  # <sjg> on the sun386i, cat works but tr -d does not!
  cat >&2 &
  r=$!

  case "$opt_s" in
  "") ;;
  [+-]*) eval `echo $opt_s | sed 's,^\(.\)\(.*\),o=\1 opt_s=\2,'`
      set -- $opt_s $o $opt_s
      ;;
  *)  set -- $opt_s - $opt_s;;
  esac

  for i in Y V $*
  do
    test "x$opt_v" = x || echo $N "$opt_p$i$opt_V$C" >&2
    case "$i" in
    KKK)			# special, tells UPS to turn off
				# the extra delay is important
      echo $N K$C; sleep 2; echo $N K$C; sleep 1; echo $N K$C;;
    *)
      echo $N $i$C;;
    esac
    sleep $s
  done
  kill -1 $r > /dev/null 2>&1 < /dev/null
  kill -9 $r > /dev/null 2>&1 < /dev/null
) < $opt_f 2>&1 > $opt_f | egrep -v 'Hangup|^$' | tr -d '!$'
